﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using TreeMapDemo.Data.Model;

namespace TreeMapDemo.Data
{
    public class DataLoader
    {
        static double _maxChangeValue = 0;
        static double _minChangeValue = 0;
        static int _minYear = 1991;
        public static List<Region> Import(int year)
        {
            //Gets data for percentage change in values and the actual values
            var perChangeValues = Resources.Resources.RelativeData_CSV.TrimEnd().Split('\n');
            var actualValues = Resources.Resources.ActualData_CSV.TrimEnd().Split('\n');  

            List<Region> regions = new List<Region>();
            List<double> perChanges = new List<double>();

            Region reg;
            for (int i = 1; i < perChangeValues.Length; i++)
            {
                //Gets fields in record
                string[] perChangeColumns = perChangeValues[i].TrimEnd().Split(',');
                string[] actualValueColumns = actualValues[i].TrimEnd().Split(',');

                //Gets region name field
                string regName = perChangeColumns[2] as string;

                //Gets list of countries for region
                var country = new Country()
                {
                    Name = perChangeColumns[0] as string,
                    Code = perChangeColumns[1] as string,
                };
                country.ChangeInValue = Convert.ToDouble(perChangeColumns[(year - _minYear) + 5]);
                country.EmissionValue = Convert.ToDouble(actualValueColumns[(year - _minYear) + 5]);

                perChanges.Add(country.ChangeInValue);

                //Checks if region already exists in region collection
                reg = regions.FirstOrDefault(item => item.Name == regName);
                if(reg == null)
                {
                    reg = new Region { Name = regName };
                    regions.Add(reg);
                }
                reg.CountryList.Add(country);

            }

            var arr = perChanges.OrderBy(x => x).ToArray();
            _maxChangeValue = arr[(int)(arr.Length * 0.95)];  //95th percentile in % change
            _minChangeValue = arr[(int)(arr.Length * 0.05)];  //5th percentile in % change

            return regions;
        }
        public static double GetMax()
        {
            return _maxChangeValue;
        }
        public static double GetMin()
        {
            return _minChangeValue;
        }
    }
}
