﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using TreeMapDemo.Data;
using TreeMapDemo.Data.Model;

namespace TreeMapDemo
{
    public partial class MainWindow : Window
    {
        int _minYear, _maxYear;
        double _maxValue, _minValue;
        public MainWindow()
        {
            InitializeComponent();
            _minYear = 1991;
            _maxYear = 2012;
        }
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            //Fills year combo-box for making selection
            for (int i = _maxYear; i >= _minYear; i--)
            {
                cbYear.Items.Add(i);
            }
            cbYear.SelectionChanged += CbYear_SelectionChanged;

            //Sets style for FlexChart Header
            C1.WPF.Chart.ChartStyle headertyle = new C1.WPF.Chart.ChartStyle();
            headertyle.FontSize = 24;
            chartTreeMap.HeaderStyle = headertyle;

            //Extracts data for selected year
            chartTreeMap.ItemsSource = DataLoader.Import(Convert.ToInt32(cbYear.SelectedValue));

            //Gets maximum and minimum change in emission value among all the countries 
            _maxValue = DataLoader.GetMax();
            _minValue = DataLoader.GetMin();
        }
        private void CbYear_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            //Changes data according to selected year
            var selectedYearData = DataLoader.Import(Convert.ToInt32(cbYear.SelectedValue));
            chartTreeMap.ItemsSource = selectedYearData;

            //Updates Data Min/Max
            _maxValue = DataLoader.GetMax();
            _minValue = DataLoader.GetMin();
        }
        private void chartTreeMap_NodeRendering(object sender, C1.WPF.Chart.RenderNodeEventArgs e)
        {
            if (!e.IsTitle)
            {
                var country = e.Item as Country;
                var value = country.ChangeInValue;
                var brsh = new SolidColorBrush();

                //showing increase in %change with red and decrease in %change with green
                //if (value > 0)
                //{
                //    brsh.Color = Colors.Red;
                //}
                //else if (value < 0)
                //{
                //    brsh.Color = Colors.Green;
                //}
                //else
                //{
                //    brsh.Color = Colors.YellowGreen;
                //}
                //e.Engine.SetFill(brsh);

                //Calculates relative brush based on value
                var brush = new SolidColorBrush(InterpolateColor(value));
                e.Engine.SetFill(brush);
            }
            else
                e.Engine.SetFill(Brushes.LightSteelBlue);
        }
        /// <summary>
        /// Gets Brush based on position of % change in Min% - Max% range
        /// </summary>
        private Color InterpolateColor(double value)
        {
            Color start, end;
            if (value <= 0)
            {
                start = Colors.YellowGreen;
                end = Colors.Green;
                value = Normalize(-value, 0,-_minValue, 0, 1);
            }
            else
            {
                start = Colors.LightGoldenrodYellow;
                end = Colors.Red;
                value = Normalize(value, 0, _maxValue, 0, 1);
            }

            double red, green, blue, alpha;

            red = Clamp(start.R + (end.R - start.R) * value);
            green = Clamp(start.G + (end.G - start.G) * value);
            blue = Clamp(start.B + (end.B - start.B) * value);
            alpha = Clamp(start.A + (end.A - start.A) * value);

            return Color.FromArgb((byte)alpha, (byte)red, (byte)green, (byte)blue);
        }
        /// <summary>
        /// Calculates value with respect to range 0-1
        /// </summary>
        private double Normalize(double value,double oldMin,double oldMax,double newMin=-1,double newMax=1)
        {
            return newMin + ((value - oldMin) * (newMax - newMin) / (oldMax - oldMin));
        }
        /// <summary>
        /// Maintain value between 0-255 grayscale
        /// </summary>
        public double Clamp(double value, double min = 0, double max = 255)
        {
            return value < min ? min : value > max ? max : value;
        }
    }
}